@@ -27,11 +27,17 @@ from ecogvis.functions.misc_dialogs import (CustomIntervalDialog, SelectChannels
                                             ExitDialog, HighGammaDialog, LoadHTKDialog,
                                             PeriodogramGridDialog, NoSpectrumDialog,
                                             PreprocessingDialog, NoRawDialog,
-                                            NoAudioDialog, ExistIntervalsDialog)
+                                            NoAudioDialog, ExistIntervalsDialog,
+                                            ShowSurveyDialog,
+                                            ShowElectrodesDialog, ShowTranscriptionDialog)
 from ecogvis.functions.audio_event_detection import AudioEventDetection
 from ecogvis.functions.event_related_potential import ERPDialog
 from ecogvis.functions.save_to_nwb import SaveToNWBDialog
 from ecogvis.functions.nwb_copy_file import nwb_copy_file
+from ecogvis.functions.survey_data import add_survey_data
+from ecogvis.functions.transcription_data import add_transcription_data
+from ecogvis.functions.htk_to_nwb.chang2nwb import chang2nwb
+
 
 annotationAdd_ = False
 annotationDel_ = False
@@ -178,9 +184,6 @@ class Application(QMainWindow):
         action_del_badchannel = QAction('Del Bad Channel', self)
         channels_tools_menu.addAction(action_del_badchannel)
         action_del_badchannel.triggered.connect(self.del_badchannel)
-        action_save_badchannel = QAction('Save Bad Channels', self)
-        channels_tools_menu.addAction(action_save_badchannel)
-        action_save_badchannel.triggered.connect(self.save_badchannel)
         action_spectral_decomposition = QAction('Spectral Decomposition', self)
         toolsMenu.addAction(action_spectral_decomposition)
         action_spectral_decomposition.triggered.connect(self.spectral_decomposition)
@@ -191,6 +194,34 @@ class Application(QMainWindow):
         toolsMenu.addAction(action_event_detection)
         action_event_detection.triggered.connect(self.audio_event_detection)
 
+        survey_tools_menu = toolsMenu.addMenu('Survey')
+        self.action_add_survey = QAction('Add Survey', self)
+        survey_tools_menu.addAction(self.action_add_survey)
+        self.action_add_survey.triggered.connect(self.add_survey)
+        self.action_vis_survey = QAction('Visualize Survey', self)
+        survey_tools_menu.addAction(self.action_vis_survey)
+        self.action_vis_survey.setEnabled(False)
+        self.action_vis_survey.triggered.connect(self.visualize_survey)
+
+        transcription_tools_menu = toolsMenu.addMenu('Transcription Data')
+        self.transcriptionadd_tools_menu = transcription_tools_menu.addMenu('Add Transcription Data')
+        action_add_timitsounds = QAction('TimitSounds', self)
+        self.transcriptionadd_tools_menu.addAction(action_add_timitsounds)
+        action_add_timitsounds.triggered.connect(self.add_transcription_timitsounds)
+        action_add_textgrid = QAction('TextGrid', self)
+        self.transcriptionadd_tools_menu.addAction(action_add_textgrid)
+        action_add_textgrid.triggered.connect(self.add_transcription_textgrid)
+        action_add_mocha = QAction('Mocha', self)
+        self.transcriptionadd_tools_menu.addAction(action_add_mocha)
+        action_add_mocha.triggered.connect(self.add_transcription_mocha)
+        self.action_vis_transcription = QAction('Visualize Transcription Data', self)
+        transcription_tools_menu.addAction(self.action_vis_transcription)
+        self.action_vis_transcription.triggered.connect(self.visualize_transcription)
+
+        self.action_vis_electrodes = QAction('Visualize Electrodes Tables', self)
+        toolsMenu.addAction(self.action_vis_electrodes)
+        self.action_vis_electrodes.triggered.connect(self.visualize_electrodes)
+
         helpMenu = mainMenu.addMenu('Help')
         action_about = QAction('About', self)
         helpMenu.addAction(action_about)
@@ -251,8 +282,6 @@ class Application(QMainWindow):
         self.push3_1.clicked.connect(self.add_badchannel)
         self.push3_2 = QPushButton('Del')
         self.push3_2.clicked.connect(self.del_badchannel)
-        self.push3_3 = QPushButton('Save')
-        self.push3_3.clicked.connect(self.save_badchannel)
 
         # Get channel by brain region
         self.push4_0 = QPushButton('Select regions')
@@ -284,7 +313,6 @@ class Application(QMainWindow):
         grid1.addWidget(qlabelBadChannels, 4, 0, 1, 6)
         grid1.addWidget(self.push3_1, 5, 0, 1, 2)
         grid1.addWidget(self.push3_2, 5, 2, 1, 2)
-        grid1.addWidget(self.push3_3, 5, 4, 1, 2)
         grid1.addWidget(self.push4_0, 6, 0, 1, 3)
         grid1.addWidget(self.push5_0, 6, 3, 1, 3)
         grid1.addWidget(self.push6_0, 7, 0, 1, 3)
@@ -455,6 +483,7 @@ class Application(QMainWindow):
         if filename is None:  # Opens new file dialog
             filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.nwb)")
         if os.path.isfile(filename):
+            self.model.io.close()
             self.source_path = Path(filename)
             # Reset file specific variables on GUI
             self.combo3.setCurrentIndex(self.combo3.findText('raw'))
@@ -480,8 +509,16 @@ class Application(QMainWindow):
         """
         w = LoadHTKDialog(parent=self)
         if w.value == 1 and w.htk_config:
-            self.source_path = Path(w.htk_config['ecephys_path'])
+            self.model.io.close()
+            htk_source_path = Path(w.htk_config['ecephys_path'])
             self.metadata = w.htk_config['metadata']
+            # Run conversion
+            _, source_path, _, _ = chang2nwb(
+                blockpath=htk_source_path,
+                save_to_file=True,
+                htk_config=w.htk_config,
+            )
+            self.source_path = source_path.absolute()
             # Reset file specific variables on GUI
             self.combo3.setCurrentIndex(self.combo3.findText('raw'))
             self.combo4.clear()
@@ -497,7 +534,7 @@ class Application(QMainWindow):
             self.win2.clear()
             self.win3.clear()
             # Rebuild the model
-            self.model = TimeSeriesPlotter(par=self, htk_config=w.htk_config)
+            self.model = TimeSeriesPlotter(par=self)
 
     def save_file(self):
         """
@@ -508,12 +545,14 @@ class Application(QMainWindow):
         """
         save_dialog = SaveToNWBDialog(parent=self)
         if save_dialog.value:
+            print('Copying content to new nwb file, please wait...')
             nwb_copy_file(
-                old_file=self.source_path,
+                old_file=self.model.nwb,
                 new_file=save_dialog.newfile,
                 cp_objs=save_dialog.cp_objs,
                 save_to_file=True
             )
+            print('File successfully copied!')
 
     def change_session(self):
         """Changes session name."""
@@ -548,10 +587,10 @@ class Application(QMainWindow):
                 for elem in ch_str:
                     if '-' in elem:   # if given a range e.g. 7-12
                         elem_lims = elem.split('-')
-                        seq = range(int(elem_lims[0]) - 1, int(elem_lims[1]))
+                        seq = range(int(elem_lims[0]), int(elem_lims[1]) + 1)
                         ch_list.extend(seq)
                     else:             # if given a single value
-                        ch_list.append(int(elem) - 1)
+                        ch_list.append(int(elem))
                 self.model.BadChannelAdd(ch_list=ch_list)
             except Exception as ex:
                 print(str(ex))
@@ -568,18 +607,14 @@ class Application(QMainWindow):
                 for elem in ch_str:
                     if '-' in elem:   # if given a range e.g. 7-12
                         elem_lims = elem.split('-')
-                        seq = range(int(elem_lims[0]) - 1, int(elem_lims[1]))
+                        seq = range(int(elem_lims[0]), int(elem_lims[1]) + 1)
                         ch_list.extend(seq)
                     else:             # if given a single value
-                        ch_list.append(int(elem) - 1)
+                        ch_list.append(int(elem))
                 self.model.BadChannelDel(ch_list=ch_list)
             except Exception as ex:
                 print(str(ex))
 
-    def save_badchannel(self):
-        """Saves bad channels to file."""
-        self.model.BadChannelSave()
-
     def spectral_decomposition(self):
         """Opens Spectral decomposition dialog."""
         w = SpectralChoiceDialog(self)
@@ -589,12 +624,8 @@ class Application(QMainWindow):
     def event_related_potential(self):
         """Opens ERP window."""
         open_dialog = True
-        try:  # Checks if file contains proper intervals information
-            speaker_start_times = self.model.nwb.intervals['TimeIntervals_speaker']['start_time'].data
-            speaker_stop_times = self.model.nwb.intervals['TimeIntervals_speaker']['stop_time'].data
-            mic_start_times = self.model.nwb.intervals['TimeIntervals_mic']['start_time'].data
-            mic_stop_times = self.model.nwb.intervals['TimeIntervals_mic']['stop_time'].data
-        except:  # if it fails, raises a no intervals data message
+        # if no intervals exist, raise a no intervals data message
+        if len(self.model.nwb.intervals)==0:
             NoTrialsDialog()
             open_dialog = False
         try:  # Tries to load High Gamma and intervals data
@@ -611,16 +642,99 @@ class Application(QMainWindow):
         # Test if trials already exist
         if 'TimeIntervals_speaker' not in self.model.nwb.intervals:
             # Test if file contains audio signals
-            # if ('microphone' in self.model.nwb.acquisition) and ('speaker1' in self.model.nwb.stimulus):
-            if any(name in self.model.nwb.acquisition for name in ['anin4', 'microphone']) and ('speaker1' in self.model.nwb.stimulus):
-                w = AudioEventDetection(parent=self)
-                if w.value == 1:  # Trial times were detected
-                    self.model.refresh_file()  # re-opens the file, now with new data
+            if any(name in self.model.nwb.stimulus for name in ['speaker1', 'speaker2']):
+                AudioEventDetection(parent=self)
             else:
                 NoAudioDialog()
         else:
             ExistIntervalsDialog()
 
+    def add_survey(self):
+        """Add survey data from .mat file to current nwb file."""
+        # Open file dialog
+        path_file, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.mat)")
+        if os.path.isfile(path_file):
+            add_survey_data(nwbfile=self.model.nwb, path_survey_file=path_file)
+            self.action_vis_survey.setEnabled(True)
+            self.action_add_survey.setEnabled(False)
+            # Write changes to NWB file
+            self.model.io.write(self.model.nwb)
+
+    def visualize_survey(self):
+        """Visualize survey data in current nwb file."""
+        # Test if current nwb file contains Survey table
+        if 'behavior' in self.model.nwb.processing:
+            list_surveys = [v for v in self.model.nwb.processing['behavior'].data_interfaces.values()
+                            if v.neurodata_type == 'SurveyTable']
+            if len(list_surveys) > 0:
+                ShowSurveyDialog(nwbfile=self.model.nwb)
+
+    def add_transcription_timitsounds(self):
+        """Add TimitSounds transcription data to current nwb file."""
+        dir_path = QFileDialog.getExistingDirectory(self, 'Open TimitSounds dir', '', QtGui.QFileDialog.ShowDirsOnly)
+        if os.path.isdir(dir_path):
+            _ = add_transcription_data(
+                nwbfile=self.model.nwb,
+                path_transcription=dir_path,
+                tr_type='timitsounds'
+            )
+            self.action_vis_transcription.setEnabled(True)
+            self.transcriptionadd_tools_menu.setEnabled(False)
+            # Write changes to NWB file
+            self.model.io.write(self.model.nwb)
+            print('Transcription data added successfully!')
+
+    def add_transcription_mocha(self):
+        """Add Mocha transcription data to current nwb file."""
+        dir_path = QFileDialog.getExistingDirectory(self, 'Open Mocha dir', '', QtGui.QFileDialog.ShowDirsOnly)
+        if os.path.isdir(dir_path):
+            nwbfile = add_transcription_data(
+                nwbfile=self.model.nwb,
+                path_transcription=dir_path,
+                tr_type='mocha',
+                subject_id=self.model.subject_id,  # 'EC118',
+                session_id='B' + self.model.block,  # 'B6'
+            )
+            if nwbfile is None:
+                print('No transcription data for {}_B{}'.format(self.model.subject_id, self.model.block))
+            else:
+                self.action_vis_transcription.setEnabled(True)
+                self.transcriptionadd_tools_menu.setEnabled(False)
+                # Write changes to NWB file
+                self.model.nwb = nwbfile
+                self.model.io.write(self.model.nwb)
+                print('Transcription data added successfully!')
+
+    def add_transcription_textgrid(self):
+        """Add TextGrid transcription data to current nwb file."""
+        filename, _ = QFileDialog.getOpenFileName(None, 'Open file', '', "(*.TextGrid)")
+        if os.path.isfile(filename):
+            add_transcription_data(
+                nwbfile=self.model.nwb,
+                path_transcription=filename,
+                tr_type='textgrid'
+            )
+            self.action_vis_transcription.setEnabled(True)
+            self.transcriptionadd_tools_menu.setEnabled(False)
+            # Write changes to NWB file
+            self.model.io.write(self.model.nwb)
+            print('Transcription data added successfully!')
+
+    def visualize_transcription(self):
+        """Visualize transcription behavioral data in current nwb file."""
+        # # Test if current nwb file contains Survey table
+        # if 'behavior' in self.model.nwb.processing:
+        #     list_surveys = [v for v in self.model.nwb.processing['behavior'].data_interfaces.values()
+        #                     if v.neurodata_type == 'SurveyTable']
+        #     if len(list_surveys) > 0:
+        #         ShowSurveyDialog(nwbfile=self.model.nwb)
+        ShowTranscriptionDialog(nwbfile=self.model.nwb)
+        pass
+
+    def visualize_electrodes(self):
+        """Visualize electrodes tables in current nwb file."""
+        ShowElectrodesDialog(parent=self)
+
     def about(self):
         """About dialog."""
         msg = QMessageBox()
@@ -754,16 +868,19 @@ class Application(QMainWindow):
         self.active_mode = 'default'
         self.reset_buttons()
         # Dialog to choose channels from specific brain regions
-        w = SelectChannelsDialog(self.model.all_regions, self.model.regions_mask)
-        all_locs = self.model.nwb.electrodes['location'][:]
+        w = SelectChannelsDialog(
+            stringlist=self.model.all_regions,
+            checked=self.model.regions_mask
+        )
+        all_locs = self.model.electrodes_table['location'][self.model.electrical_series_channel_ids]
         self.model.channels_mask = np.zeros(len(all_locs))
         for loc in w.choices:
             self.model.channels_mask += all_locs == np.array(loc)
         # Indices of channels from chosen regions
         self.model.channels_mask_ind = np.where(self.model.channels_mask)[0]
-        self.model.nChTotal = len(self.model.channels_mask_ind)
+        self.model.n_channels_total = len(self.model.channels_mask_ind)
         # Reset channels span control
-        self.model.lastCh = np.minimum(16, self.model.nChTotal)
+        self.model.lastCh = np.minimum(16, self.model.n_channels_total)
         self.model.firstCh = 1
         self.model.nChToShow = self.model.lastCh - self.model.firstCh + 1
         self.qline0.setText(str(self.model.lastCh))
