@@ -14,7 +14,9 @@ from pathlib import Path
 import numpy as np
 import pandas as pd
 from hdmf.backends.hdf5 import H5DataIO
-from nwbext_ecog import ECoGSubject
+from ndx_ecog import ECoGSubject
+from ndx_bipolar_scheme import BipolarSchemeTable, EcephysExt
+from pynwb.file import DynamicTableRegion
 from pynwb import NWBFile, TimeSeries, get_manager, NWBHDF5IO
 from pynwb.ecephys import ElectricalSeries, LFP
 import scipy.io as sio
@@ -242,7 +244,8 @@ def chang2nwb(blockpath, out_file_path=None, save_to_file=False, htk_config=None
             anin3: {present: False, name: 'speaker2', type: 'stimulus'},
             anin4: {present: False, name: 'custom', type: 'acquisition'},
             metadata: metadata,
-            electrodes_file: electrodes_file
+            electrodes_file: electrodes_file,
+            bipolar_file: bipolar_file
         }
 
     Returns
@@ -294,10 +297,10 @@ def chang2nwb(blockpath, out_file_path=None, save_to_file=False, htk_config=None
         n_electrodes = nwbfile.electrodes[:].shape[0]
         all_elecs = list(range(n_electrodes))
         elecs_region = nwbfile.create_electrode_table_region(
-            all_elecs,
-            'ECoG electrodes on brain'
+            region=all_elecs,
+            description='ECoG electrodes on brain'
         )
-    else:  # Get electrodes info from metadata file
+    else:
         ecephys_dict = {
             'Device': [{'name': 'auto_device'}],
             'ElectricalSeries': [{'name': 'ECoG', 'description': 'description'}],
@@ -333,7 +336,7 @@ def chang2nwb(blockpath, out_file_path=None, save_to_file=False, htk_config=None
         for elec_counter in range(n_electrodes):
             bad = elec_counter in bad_elecs_inds
             nwbfile.add_electrode(
-                id=elec_counter + 1,
+                id=elec_counter,
                 x=np.nan,
                 y=np.nan,
                 z=np.nan,
@@ -351,10 +354,63 @@ def chang2nwb(blockpath, out_file_path=None, save_to_file=False, htk_config=None
 
         all_elecs = list(range(n_electrodes))
         elecs_region = nwbfile.create_electrode_table_region(
-            all_elecs,
-            'ECoG electrodes on brain'
+            region=all_elecs,
+            description='ECoG electrodes on brain'
         )
 
+    # Get Bipolar table from file
+    if htk_config['bipolar_file'] is not None:
+        df = pd.read_csv(htk_config['bipolar_file'], index_col='id', sep='\t')
+
+        # Create bipolar scheme table
+        bipolar_scheme_table = BipolarSchemeTable(
+            name='bipolar_scheme_table',
+            description='desc'
+        )
+
+        # Columns for bipolar scheme - all anodes and cathodes within the same
+        # bipolar row are considered to have the same group and location
+        bipolar_scheme_table.add_column(
+            name='group_name',
+            description='electrode group name'
+        )
+        bipolar_scheme_table.add_column(
+            name='location',
+            description='electrode location'
+        )
+
+        # Iterate over anode / cathode rows
+        for i, r in df.iterrows():
+            if isinstance(r['anodes'], str):
+                anodes = [int(a) for a in r['anodes'].split(',')]
+            else:
+                anodes = [int(r['anodes'])]
+            if isinstance(r['cathodes'], str):
+                cathodes = [int(a) for a in r['cathodes'].split(',')]
+            else:
+                cathodes = [int(r['cathodes'])]
+            bipolar_scheme_table.add_row(
+                anodes=anodes,
+                cathodes=cathodes,
+                group_name=nwbfile.electrodes['group_name'][anodes[0]],
+                location=nwbfile.electrodes['location'][anodes[0]]
+            )
+
+        bipolar_scheme_table.anodes.table = nwbfile.electrodes
+        bipolar_scheme_table.cathodes.table = nwbfile.electrodes
+
+        # Creates bipolar table region
+        elecs_region = DynamicTableRegion(
+            name='electrodes',
+            data=np.arange(0, df.shape[0]),
+            description='desc',
+            table=bipolar_scheme_table
+        )
+
+        ecephys_ext = EcephysExt(name='ecephys_ext')
+        ecephys_ext.bipolar_scheme_table = bipolar_scheme_table
+        nwbfile.add_lab_meta_data(ecephys_ext)
+
     # Stores HTK electrophysiology data as raw, preprocessed or high gamma
     if htk_config['ecephys_type'] == 'raw':
         ecog_es = ElectricalSeries(name='ECoG',
